;;;Code for the pure manipulation model of the algebra task from
;;;
;;;Danker, J. F. & Anderson, J. R. The roles of prefrontal and posterior parietal cortex in algebra problem-solving: 
;;;A case of using cognitive modeling to inform neuroimaging data.
;;;
;;;This model has to be run using Macintosh Common Lisp and ACT-R 6.0. It might work with other Lisps, but probably
;;;needs some adjustments for that.
;;;
;;;
;;;
;;;These are the four relevant example problems, one for each condition (Transformation x Retrieval).
;;;To run the model for one of the four conditions, paste one of the following into the Lisp window with 
;;;ACT-R running and the model loaded.
;;;
;;;Low Transformation, Low Retrieval
;;;(present-trial (make-trial :trans 0 :ret 0 :delay 0 :equation "x=a+b" :a "1-0" :b "1-0" :answer-op + :answer-x 2))
;;;
;;;Low Transformation, High Retrieval
;;;(present-trial (make-trial :trans 0 :ret 1 :delay 0 :equation "x=a+b" :a "2-1" :b "2-1" :answer-op + :answer-x 2))
;;;
;;;High Transformation, Low Retrieval
;;;(present-trial (make-trial :trans 1 :ret 0 :delay 0 :equation "x-b=a" :a "1-0" :b "1-0" :answer-op + :answer-x 2))
;;;
;;;High Transformation, High Retrieval
;;;(present-trial (make-trial :trans 1 :ret 1 :delay 0 :equation "x-b=a" :a "2-1" :b "2-1" :answer-op + :answer-x 2))

;;EXPERIMENT
(defvar *trials*)
(defvar *results*)
(defvar *start-time*)
(defvar *show-window*)

(defstruct trial trans ret delay equation a b answer-op answer-x)
(defstruct response trans ret delay correct-op rt-op correct-x rt-x)

(defun present-trial (trial)
  (let ((window (open-exp-window "Algebra Experiment" :visible nil)))
    (clear-exp-window)

    (reset)

    (install-device window)

    ;;present equation and cues
    ;;equation
    (dotimes (i (array-dimension (trial-equation trial) 0))
      (add-text-to-exp-window :text (string (char (trial-equation trial) i)) :x (round (-(*(/ 300 (array-dimension (trial-equation trial) 0)) (+ i 1)) (/ (/ 300 (array-dimension (trial-equation trial) 0)) 2)))  :y 150)
    )
    ;;+-*/ cues
    (add-text-to-exp-window :text "+" :x 75 :y 275)
    (add-text-to-exp-window :text "-" :x 125 :y 275)
    (add-text-to-exp-window :text "*" :x 175 :y 275)
    (add-text-to-exp-window :text "/" :x 225 :y 275)

    (proc-display :clear t)

    (setf *start-time* (get-time))

    ;;run model until it responds to the equation with +-*/
    (run 10)

    ;;clear the window
    (clear-exp-window)
    
    ;;present values for a and b with cues
    ;;a
    (add-text-to-exp-window :text "a" :x (round (-(*(/ 300 (+ (array-dimension (trial-a trial) 0) 2)) 1) (/ (/ 300 (+ (array-dimension (trial-a trial) 0) 2)) 2))) :y 125)
    (add-text-to-exp-window :text "=" :x (round (-(*(/ 300 (+ (array-dimension (trial-a trial) 0) 2)) 2) (/ (/ 300 (+ (array-dimension (trial-a trial) 0) 2)) 2))) :y 125)
    (dotimes (i (array-dimension (trial-a trial) 0))
      (add-text-to-exp-window :text (string (char (trial-a trial) i)) :x (round (-(*(/ 300 (+ (array-dimension (trial-a trial) 0) 2)) (+ i 3)) (/ (/ 300 (+ (array-dimension (trial-a trial) 0) 2)) 2))) :y 125)
    )
    ;;b
    (add-text-to-exp-window :text "b" :x (round (-(*(/ 300 (+ (array-dimension (trial-b trial) 0) 2)) 1) (/ (/ 300 (+ (array-dimension (trial-b trial) 0) 2)) 2))) :y 175)
    (add-text-to-exp-window :text "=" :x (round (-(*(/ 300 (+ (array-dimension (trial-b trial) 0) 2)) 2) (/ (/ 300 (+ (array-dimension (trial-b trial) 0) 2)) 2))) :y 175)
    (dotimes (i (array-dimension (trial-b trial) 0))
      (add-text-to-exp-window :text (string (char (trial-b trial) i)) :x (round (-(*(/ 300 (+ (array-dimension (trial-b trial) 0) 2)) (+ i 3)) (/ (/ 300 (+ (array-dimension (trial-b trial) 0) 2)) 2))) :y 175)
    ) 
    ;;2468 cues
    (add-text-to-exp-window :text "2" :x 75 :y 275)
    (add-text-to-exp-window :text "4" :x 125 :y 275)
    (add-text-to-exp-window :text "6" :x 175 :y 275)
    (add-text-to-exp-window :text "8" :x 225 :y 275)
    
    (proc-display :clear t)

    (setf *start-time* (get-time))

    (run 10)

    (clear-exp-window)

    window))


;;MODEL
(clear-all)

(define-model jared-algebra

;;The 600 ms lf is taken from Anderson, Reder, & Lebiere (1996)
(sgp :esc t :lf .6 :trace-detail medium :show-focus t :imaginal-delay .2)

;;This sets the default location conveniently to the 1 in a = 1 - 0
(set-visloc-default :screen-y lowest :screen-x (within 130 160))

(chunk-type problem equals a oper b x)
(chunk-type problem-state larg1 loper1 larg2 rarg1 roper1 rarg2)
(chunk-type math-fact arg1 oper arg2 ans)
(chunk-type symbol string type inverted-by key)

(add-dm
  (operator isa chunk)
  (number isa chunk)
  (goal isa problem)
  (image isa problem-state)
  (+ isa symbol string "+" type operator inverted-by "-" key "j")
  (* isa symbol string "*" type operator inverted-by "/" key "k")
  (- isa symbol string "-" type operator inverted-by "+" key "l")
  (/ isa symbol string "/" type operator inverted-by "*" key ";")
  (two isa symbol string "2" type number key "j")
  (four isa symbol string "4" type number key "k")
  (six isa symbol string "6" type number key "l")
  (eight isa symbol string "8" type number key ";")
  ;;The full array of necessary math-facts can be produced using mkmathfacts.m
  ;;for simplicity only two of hundreds are in this model
  (1plus1 isa math-fact arg1 "1" oper "+" arg2 "1" ans "2")
  (2minus1 isa math-fact arg1 "2" oper "-" arg2 "1" ans "1")
)
 

;;FINDS THE FIRST ELEMENT ON THE SCREEN - FIRST PART OF TRIAL
(P find-first-element
  =goal>
    ISA        problem
    oper       nil
  ?visual-location>
    buffer     unrequested
  ?visual>
    buffer     empty
==>  
  +visual-location>
    ISA        visual-location
    screen-y   lowest
    screen-x   lowest
)

;;FINDS THE NEXT ELEMENT IN A ROW, SCANS FROM LEFT TO RIGHT - FIRST PART OF TRIAL
(P find-next-element
  =goal>
    ISA        problem
    oper       nil
  ?visual-location>
    buffer     empty
  - state      error
==>  
  +visual-location>
    ISA        visual-location
  > screen-x   current
    nearest    current
  -visual>
)

;;SHIFTS ATTENTION TO THE LOCATION OF AN ELEMENT
(P attend-element
  =goal>
    ISA        problem
  =visual-location>
    ISA        visual-location
  ?visual-location>
    buffer     requested
  ?visual>
    state      free
    buffer     empty
==>
  +visual>
    ISA        move-attention
    screen-pos =visual-location
  =visual-location>
)

;;ENCODES THE FIRST ARGUMENT ON THE LEFT INTO GOAL BUFFER
(P encode-larg1
  =goal>
    ISA        problem
    oper       nil
    equals     nil
  =visual>
    ISA        text
    value      =larg1
  =visual-location>
    ISA        visual-location
  ?imaginal>
    buffer     empty
    state      free
==>
  +imaginal>
    isa        problem-state
    larg1      =larg1
  =visual>
)

;;ENCODES THE FIRST OPERATOR ON THE LEFT INTO GOAL BUFFER
(P encode-loper1
  =goal>
    ISA        problem
    oper       nil
    equals     nil
  =imaginal>
    isa        problem-state
    larg1      =larg1
    loper1     nil
  =visual>
    ISA        text
    value      =loper1
  - value      "="
  =visual-location>
    ISA        visual-location
  ?imaginal>
    state      free
==>
  =imaginal>
    loper1     =loper1
  +imaginal>   =imaginal
  =visual>
)

;;ENCODES THE SECOND ARGUMENT ON THE LEFT INTO GOAL BUFFER
(P encode-larg2
  =goal>
    ISA        problem
    oper       nil
    equals     nil
  =imaginal>
    isa        problem-state
    larg1      =larg1
    loper1     =loper1
    larg2      nil
  =visual>
    ISA        text
    value      =larg2
  =visual-location>
    ISA        visual-location
  ?imaginal>
    state      free
==>
  =imaginal>
    larg2      =larg2
  +imaginal>   =imaginal
  =visual>
)

;;ENCODES THE LOCATION OF THE EQUALS SIGNS INTO GOAL BUFFER
(P encode-equals
  =goal>
    ISA        problem
    oper       nil
    equals     nil
  =visual>
    ISA        text
    screen-pos =visual-location
    value      "="
  =visual-location>
    ISA        visual-location
==>
  =goal>
    equals     =visual-location
  =visual>
)

;;ENCODES THE FIRST ARGUMENT ON THE RIGHT INTO IMAGINAL BUFFER - FIRST PART OF TRIAL
(P encode-rarg1
  =goal>
    ISA        problem
    oper       nil
    equals     =equals
  =imaginal>
    ISA        problem-state
    rarg1      nil  
  =visual>
    ISA        text
    value      =rarg1
  =visual-location>
    ISA        visual-location
  ?imaginal>
    state      free
==>
  =imaginal>
    rarg1      =rarg1
  +imaginal>   =imaginal
  =visual>
)

;;ENCODES THE FIRST OPERATOR ON THE RIGHT INTO IMAGINAL BUFFER
(P encode-roper1
  =goal>
    ISA        problem
    oper       nil
  =visual>
    ISA        text
    value      =roper1
  =visual-location>
    ISA        visual-location
  =imaginal>
    ISA        problem-state
    rarg1      =rarg1
    roper1     nil
  ?imaginal>
    state      free
==>
  =imaginal>
    roper1     =roper1
  +imaginal>   =imaginal
  =visual>
)

;;ENCODES THE SECOND ARGUMENT ON THE RIGHT INTO IMAGINAL BUFFER - FIRST PART OF TRIAL
(P encode-rarg2
  =goal>
    ISA        problem
    oper       nil
  =visual>
    ISA        text
    value      =rarg2
  =visual-location>
    ISA        visual-location
  =imaginal>
    ISA        problem-state
    rarg1      =rarg1
    roper1     =roper1
    rarg2      nil
  ?imaginal>
    state      free
==>
  =imaginal>
    rarg2     =rarg2
  +imaginal>  =imaginal
  =visual>
)

;;ISOLATES X
;;only works with x-b=a
(P perform-transformation-+
  =goal>
    ISA       problem
    oper      nil
  =imaginal>
    ISA       problem-state
    larg1     "x"
    loper1    "-"
    larg2     =larg2
    rarg1     =rarg1
    roper1    nil
    rarg2     nil
  ?imaginal>
    state     free
  ?visual-location>
    state     error
==>
  =imaginal>
    loper1    nil
    larg2     nil
    roper1    "+"
    rarg2     =larg2
  +imaginal>  =imaginal
)

;;RETRIEVES THE APPROPRIATE KEY RESPONSE FOR THE OPERATOR
;;STORES THE OPERATOR IN THE GOAL BUFFER FOR USE LATER
(P retrieve-key-oper
  =goal>
    ISA       problem
    oper      nil
    equals    =equals
  =imaginal>
    isa       problem-state
    larg1     "x"
    loper1    nil
    rarg1     "a"
    roper1    =oper
    rarg2     "b"
  ?visual-location>
    state     error
  ?retrieval>
    state     free
==>
  +retrieval>
    ISA       symbol
    string    =oper
  =goal>
    oper      =oper
  -imaginal>
)

;;MAKES THE FIRST RESPONSE, THE OPERATOR BETWEEN A AND B
(P respond-oper
  =goal>
    ISA       problem
    oper      =oper
    equals    =equals
  =retrieval>
    isa       symbol
    string    =oper
    key       =key
==>   
  +manual>
    ISA       press-key
    key       =key
  =goal>
    equals    nil
  +visual>
    isa       clear
)

;;ATTENDS THE FIRST ARGUMENT ON THE RIGHT, WHICH SHOULD'VE BEEN STUFFED INTO VISUAL-LOCATION
(P attend-minuend-a
  =goal>
    ISA        problem
    oper       =oper
  =visual-location>
    isa        visual-location
  ?visual-location>
    buffer     unrequested
  ?visual>
    state      free
    buffer     empty
==>  
  +visual>
    ISA        move-attention
    screen-pos =visual-location
  =visual-location>
)

;;ENCODES THE FIRST ARGUMENT ON THE RIGHT INTO IMAGINAL BUFFER - SECOND PART OF TRIAL
(P encode-minuend-a
  =goal>
    ISA        problem
    oper       =oper
    a          nil
  =visual>
    ISA        text
    value      =larg1
  =visual-location>
    ISA        visual-location
  ?imaginal>
    buffer     empty
    state      free
==>
  =goal>
  +imaginal>
    isa       problem-state
    larg1     =larg1
  =visual>
)

;;FINDS THE NEXT ELEMENT IN ROW A, SCANS FROM LEFT TO RIGHT - SECOND PART OF TRIAL
(P find-subtrahend-a
  =goal>
    ISA        problem
    oper       =oper
    a          nil
  =imaginal>
    isa        problem-state
    larg1      =rarg1
    larg2      nil
  ?visual-location>
    buffer     empty
  - state      error
==>  
  +visual-location>
    ISA        visual-location
  > screen-x   250
  < screen-x   300
    nearest    current
  =imaginal>
  -visual>
)

;;ENCODES THE SECOND ARGUMENT ON THE RIGHT INTO IMAGINAL BUFFER - SECOND PART OF TRIAL
(P encode-subtrahend-a
  =goal>
    ISA        problem
    oper       =oper
    a          nil
  =visual>
    ISA        text
    value      =larg2
  =visual-location>
    ISA        visual-location
  =imaginal>
    ISA        problem-state
    larg1      =larg1
    larg2      nil
  ?imaginal>
    state      free
==>
  =goal>
  =visual>
  =imaginal>
    larg2      =larg2
  +imaginal>   =imaginal
)

;;PUT THE VALUE OF A INTO THE GOAL BUFFER
(P encode-a
  =goal>
    ISA       problem
    oper      =oper
    a         nil
  =imaginal>
    ISA       problem-state
    larg1     =larg1
    larg2     "0"
==>
  =goal>
    a         =larg1
  =imaginal>
  -visual>
)

;;RETRIEVE VALUE OF A IF NECESSARY
(P retrieve-sub-fact-a
  =goal>
    ISA       problem
    oper      =oper
    a         nil
  =imaginal>
    ISA       problem-state
    larg1     =larg1
    larg2     =larg2
  - larg2     "0"
  ?retrieval>
    state     free
    buffer    empty
==>
  =imaginal>
  +retrieval>
    ISA       math-fact
    arg1      =larg1
    oper      "-"
    arg2      =larg2
)

;;PUT THE VALUE OF A INTO THE GOAL BUFFER
(P harvest-sub-fact-a
  =goal>
    ISA       problem
    oper      =oper
    a         nil
  =imaginal>
    ISA       problem-state
  =retrieval>
    ISA       math-fact
    ans       =ans
==>
  =goal>
    a         =ans
  =imaginal>
  -visual>
)

;;WHEN YOU GET STUCK AT THE END OF A ROW, MOVES DOWN TO THE NEXT ROW
(P find-minued-b
  =goal>
    ISA        problem
    oper       =oper
    a          =a
    b          nil
  =imaginal>
    isa        problem-state
    larg1      =larg1
    larg2      =larg2
    rarg1      nil
  ?visual-location>
    buffer     empty
  ?visual>
    buffer     empty
==>
  +visual-location>  
    ISA        visual-location
  > screen-y   135
  < screen-y   285
  > screen-x   130
  < screen-x   160
  =imaginal>
)

;;ENCODES THE FIRST ARGUMENT ON THE RIGHT INTO IMAGINAL BUFFER - SECOND PART OF TRIAL
(P encode-minuend-b
  =goal>
    ISA        problem
    oper       =oper
    a          =a
    b          nil
  =visual>
    ISA        text
    value      =rarg1
  =visual-location>
    ISA        visual-location
  =imaginal>
    isa        problem-state
    larg1      =larg1
    larg2      =larg2
    rarg1      nil
  ?imaginal>
    state      free
==>
  =goal>
  =imaginal>
    rarg1      =rarg1
  +imaginal>   =imaginal 
  =visual>
)

;;FINDS THE NEXT ELEMENT IN ROW B, SCANS FROM LEFT TO RIGHT - SECOND PART OF TRIAL
(P find-subtrahend-b
  =goal>
    ISA        problem
    oper       =oper
  =imaginal>
    isa        problem-state
    larg1      =larg1
    rarg1      =rarg1
    rarg2      nil
  ?visual-location>
    buffer     empty
  - state      error
==>  
  +visual-location>
    ISA        visual-location
  > screen-x   250
  < screen-x   300
    nearest    current
  =imaginal>
  -visual>
)

;;ENCODES THE SECOND ARGUMENT ON THE RIGHT INTO IMAGINAL BUFFER - SECOND PART OF TRIAL
(P encode-subtrahend-b
  =goal>
    ISA        problem
    oper       =oper
  =visual>
    ISA        text
    value      =rarg2
  =visual-location>
    ISA        visual-location
  =imaginal>
    ISA        problem-state
    larg1      =larg1
    larg2      =larg2
    rarg1      =rarg1
    rarg2      nil
  ?imaginal>
    state      free
==>
  =goal>
  =visual>
  =imaginal>
    rarg2      =rarg2
  +imaginal>   =imaginal
)

;;PUT THE VALUE OF B INTO THE GOAL BUFFER
(P encode-b
  =goal>
    ISA       problem
    oper      =oper
    a         =a
    b         nil
  =imaginal>
    ISA       problem-state
    larg1     =larg1
    larg2     =larg2
    rarg1     =rarg1
    rarg2     "0"
==>
  =goal>
    b         =rarg1
  -imaginal>
)

;;RETRIEVE VALUE OF B IF NECESSARY
(P retrieve-sub-fact-b
  =goal>
    ISA       problem
    oper      =oper
    a         =a
    b         nil
  =imaginal>
    ISA       problem-state
    rarg1     =rarg1
    rarg2     =rarg2
  - rarg2     "0"
  ?retrieval>
    state     free
    buffer    empty
==>
  =imaginal>
  +retrieval>
    ISA       math-fact
    arg1      =rarg1
    oper      "-"
    arg2      =rarg2
)

;;PUT THE VALUE OF B INTO THE GOAL BUFFER
(P harvest-sub-fact-b
  =goal>
    ISA       problem
    oper      =oper
    a         =a
    b         nil
  =imaginal>
    ISA       problem-state
  =retrieval>
    ISA       math-fact
    ans       =ans
==>
  =goal>
    b         =ans
  -imaginal>
)

;;RETRIEVE THE VALUE OF X
(P retrieve-a-op-b
  =goal>
    ISA       problem
    a         =a
    oper      =oper
    b         =b
    x         nil
  ?retrieval>
    state     free
    buffer    empty
==>
  +retrieval>
    ISA       math-fact
    arg1      =a
    oper      =oper
    arg2      =b
)

;;PUT THE VALUE OF X INTO THE GOAL BUFFER
(P harvest-a-op-b
  =goal>
    ISA       problem
    a         =a
    oper      =oper
    b         =b
    x         nil
  =retrieval>
    ISA       math-fact
    arg1      =a
    oper      =oper
    arg2      =b
    ans       =ans
==>
  =goal>
    x         =ans
)

;;RETRIEVES THE APPROPRIATE KEY RESPONSE FOR X
;;only works for x = 2, 4, 6, or 8
(P retrieve-key-x
  =goal>
    ISA       problem
    x         =x
  ?retrieval>
    state     free
==>
  +retrieval>
    ISA       symbol
    string    =x
)

;;MAKES THE SECOND RESPONSE, THE VALUE OF X
(P respond-x
  =goal>
    ISA       problem
    x         =x
  =retrieval>
    ISA       symbol
    string    =x
    key       =key
  ?manual>
    state     free
==>
  +manual>
    ISA       press-key
    key       =key
  -goal>
)


(setf *actr-enabled-p* t)

(goal-focus goal)
)
